package mandelbrot.korrekt;

import mandelbrot.korrekt.Complex;

/**
 * Library for calculations with complex numbers. A max. error of
 * 10<sup>-5</sup> is guaranteed
 * 
 * @author D. F&auml;ckt
 * 
 */
public final class Complex {

	private double real;
	private double imag;

	/**
	 * Copy constructor. Create a copy of the complex number <code>c</code>.
	 * 
	 * @param c
	 *            the number to copy
	 */
	public Complex(Complex c) {
		this.real = c.real;
		this.imag = c.imag;
	}

	/**
	 * Create a complex number
	 * 
	 * @param real
	 *            the real part
	 * @param imag
	 *            the imag part
	 */
	public Complex(double real, double imag) {
		this.real = real;
		this.imag = imag;
	}

	/**
	 * Returns the real part of this number
	 * 
	 * @return the real part
	 */
	public double getReal() {
		return real;
	}

	/**
	 * Returns the imag part of this number
	 * 
	 * @return the imag part
	 */
	public double getImag() {
		return imag;
	}

	/**
	 * Adds the complex number <code>c</code> to <code>this</code>.
	 * <code>this</code> will be modified by the method call and returned as
	 * result. Use the copy constructor to backup the complex number.
	 * 
	 * @param c
	 *            the summand
	 * @return the result of the addition
	 */
	public Complex add(Complex c) {
		double r = real + c.real;
		double i = imag + c.imag;
		real = r;
		imag = i;
		return this;
	}

	/**
	 * Subtracts the complex number <code>c</code> from <code>this</code>
	 * <code>this</code> will be modified by the method call and returned as
	 * result. Use the copy constructor to backup the complex number.
	 * 
	 * @param c
	 *            the subtrahend
	 * @return the result of the subtraction
	 */
	public Complex sub(Complex c) {
		double r = real - c.real;
		double i = imag - c.imag;
		real = r;
		imag = i;
		return this;
	}

	/**
	 * Multiplies the complex number <code>c</code> with <code>this</code>
	 * <code>this</code> will be modified by the method call and returned as
	 * result. Use the copy constructor to backup the complex number.
	 * 
	 * @param c
	 *            the multiplier
	 * @return the result of the multiplication
	 */
	public Complex mul(Complex c) {
		double r = real * c.real - imag * c.imag;
		double i = imag * c.real + real * c.imag;
		real = r;
		imag = i;
		return this;
	}

	/**
	 * Squares <code>this</code>
	 * <code>this</code> will be modified by the method call and returned as
	 * result. Use the copy constructor to backup the complex number.
	 * 
	 * @return the square of <code>this</code>
	 */
	public Complex square() {
		double r = real * real - imag * imag;
		double i = 2 * real * imag;
		real = r;
		imag = i;
		return this;
	}

	/**
	 * Calculates the square of the modulus of <code>this</code>.
	 * 
	 * @return the square of the modulus
	 */
	public double modsq() {
		return real * real + imag * imag;
	}

	/**
	 * Creates a textual representation of this <code>this</code>:
	 *  (&lt;real part&gt; + * &lt;imag part&gt;i)
	 * @return the textual representation 
	 */
	@Override
	public String toString() {
		return real + " + " + imag + "i";
	}
}
