package mandelbrot.korrekt;

import mandelbrot.korrekt.Complex;

/**
 * Sequential implementation of the Mandelbrot set computation. Multiple
 * OOMandelbrot instances with different dimensions may exist.
 * 
 * @author K. Putt
 */
public final class OOMandelbrot {
	private static final double MAGNITUDE = 4.0;
	private static final int MAX_ITER = 1500;
	private static final double TOP_LEFT_REAL = -2;
	private static final double TOP_LEFT_IMAG = 1.5;
	private static final double BOTTOM_RIGHT_REAL = 1;
	private static final double BOTTOM_RIGHT_IMAG = -1.5;

	private int width;
	private int height;

	/**
	 * The created algorithm will compute a Mandelbrot set with the given width
	 * and height.
	 * 
	 * @param width
	 *            the width of the Mandelbrot set
	 * @param height
	 *            the height of the Mandelbrot set
	 */
	public OOMandelbrot(int width, int height) {
		this.width = width;
		this.height = height;
	}

	/**
	 * Triggers the computation of the Mandelbrot set. It also prints the
	 * computation's duration in milliseconds to the console.
	 * 
	 * @return the Mandelbrot
	 */
	public final int[][] computeMandelbrot() {
		long start = System.nanoTime();
		int[][] temp = compute();
		System.out.print((System.nanoTime() - start) / 1000000);
		return temp;
	}

	/**
	 * Computes the Mandelbrot set.
	 * 
	 * @return the Mandelbrot
	 */
	protected final int[][] compute() {
		int[][] result = new int[width][height];
		Complex c;
		for (int x = 0; x < width; x++) {
			for (int y = 0; y < height; y++) {
				c = translate(new Complex(x, y));
				result[x][y] = computeMandelbrotPoint(c);
			}
		}
		return result;
	}

	/**
	 * Translates the complex number c using the following 
	 * computation:
	 *   new real = TOP_LEFT_REAL + real(c) / height * (DIAGONAL_SIZE_REAL)
	 *   new imag = TOP_LEFT_IMAG + imag(c) / width  * (DIAGONAL_SIZE_IMAG)
	 * @param c
	 *            complex number to translate
	 * @return the translated complex number
	 */
	protected final Complex translate(Complex c) {
		double real = TOP_LEFT_REAL + c.getReal() / height
				* (BOTTOM_RIGHT_REAL - TOP_LEFT_REAL);
		double imag = TOP_LEFT_IMAG + c.getImag() / width
				* (BOTTOM_RIGHT_IMAG - TOP_LEFT_IMAG);

		return new Complex(real, imag);
	}

	/**
	 * Computes one point of the Mandelbrot set.
	 * 
	 * @param c
	 *            the point to compute
	 * @return the number of iterations after which the computation stopped. If
	 *         this number exceeds the maximal number of iterations 0 is
	 *         returned.
	 */
	protected final int computeMandelbrotPoint(Complex c) {
		Complex z = new Complex(0, 0);
		int i = 0;
		do {
			i++;
			z = transform(z, c);
		} while (((z.modsq()) <= MAGNITUDE) && (i <= MAX_ITER));
		return i % MAX_ITER;
	}

	/**
	 * Computes one iteration of the Mandelbrot polynomial z(n+1) <- z(n)^2 + c
	 * 
	 * @param z
	 *            z
	 * @param c
	 *            c
	 * @return z(n+1)'s
	 */
	protected final Complex transform(Complex z, Complex c) {
		Complex newC = (z.square()).add(c);
		return newC;
	}

	/**
	 * @return the width of the Mandelbrot set
	 */
	public final int getWidth() {
		return width;
	}

	/**
	 * @return the height of the Mandelbrot set
	 */
	public final int getHeight() {
		return height;
	}
}
